@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * cap.c: interface to libpcap.
  *
@@ -11,12 +11,16 @@
 #include "cap.h"
 #include "conv.h"
 #include "decode.h"
+#include "hosts_db.h"
 #include "localip.h"
 
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/wait.h>
+#ifdef HAVE_SYS_FILIO_H
+# include <sys/filio.h> /* Solaris' FIONBIO hides here */
+#endif
 #include <assert.h>
 #include "err.h"
 #include <stdio.h>
@@ -43,7 +47,7 @@ static const linkhdr_t *linkhdr = NULL;
  * Init pcap.  Exits on failure.
  */
 void
-cap_init(const char *device, const char *filter)
+cap_init(const char *device, const char *filter, int promisc)
 {
    char errbuf[PCAP_ERRBUF_SIZE], *tmp_device;
    int linktype, caplen;
@@ -72,6 +76,8 @@ cap_init(const char *device, const char *filter)
       errx(1, "no handler for linktype %d", linktype);
    caplen = getcaplen(linkhdr);
    verbosef("caplen is %d", caplen);
+   if (linktype == DLT_EN10MB)
+      show_mac_addrs = 1;
 
    /* Close and re-open pcap to use the new caplen. */
    pcap_close(pcap);
@@ -79,7 +85,7 @@ cap_init(const char *device, const char *filter)
    pcap = pcap_open_live(
       tmp_device,
       caplen,      /* snaplen */
-      1,           /* promisc, FIXME */
+      promisc,
       CAP_TIMEOUT,
       errbuf);
 
@@ -91,6 +97,11 @@ cap_init(const char *device, const char *filter)
 
    free(tmp_device);
 
+   if (promisc)
+      verbosef("capturing in promiscuous mode");
+   else
+      verbosef("capturing in non-promiscuous mode");
+
    /* Set filter expression, if any. */
    if (filter != NULL)
    {
@@ -112,28 +123,13 @@ cap_init(const char *device, const char *filter)
       free(tmp_filter);
    }
 
-#ifdef HAVE_PCAP_GET_SELECTABLE_FD
-   if (pcap_setnonblock(pcap, 1, errbuf) == -1)
-      errx(1, "pcap_setnonblock(): %s", errbuf);
-
-   pcap_fd = pcap_get_selectable_fd(pcap);
-   if (pcap_fd == -1)
-      errx(1, "pcap_get_selectable_fd(): there isn't one!");
-
-   verbosef("pcap_got_selectable_fd");
-#else
-   /* hax */
    pcap_fd = pcap_fileno(pcap);
 
+   /* set non-blocking */
 { int one = 1;
    if (ioctl(pcap_fd, FIONBIO, &one) == -1)
       err(1, "ioctl(pcap_fd, FIONBIO)"); }
 
-{ struct timeval t = { 0, CAP_TIMEOUT * 1000 }; /* msec -> usec */
-   if (ioctl(pcap_fd, BIOCSRTIMEOUT, &t) == -1)
-      err(1, "ioctl(pcap_fd, BIOCSRTIMEOUT)"); }
-#endif
-
 #ifdef BIOCSETWF
 {
    /* Deny all writes to the socket */
@@ -151,6 +147,7 @@ cap_init(const char *device, const char *filter)
 #endif
 
 #ifdef BIOCLOCK
+   /* set "locked" flag (no reset) */
    if (ioctl(pcap_fd, BIOCLOCK) == -1)
       err(1, "ioctl(pcap_fd, BIOCLOCK)");
    verbosef("locked down BPF for security");
@@ -161,8 +158,17 @@ cap_init(const char *device, const char *filter)
  * Set pcap_fd in the given fd_set.
  */
 void
-cap_fd_set(fd_set *read_set, int *max_fd,
-   struct timeval *timeout, int *need_timeout)
+cap_fd_set(
+#ifdef linux
+   fd_set *read_set _unused_,
+   int *max_fd _unused_,
+   struct timeval *timeout,
+#else
+   fd_set *read_set,
+   int *max_fd,
+   struct timeval *timeout _unused_,
+#endif
+   int *need_timeout)
 {
    assert(*need_timeout == 0); /* we're first to get a shot at this */
 #ifdef linux
@@ -184,13 +190,17 @@ cap_fd_set(fd_set *read_set, int *max_fd,
  * Process any packets currently in the capture buffer.
  */
 void
-cap_poll(fd_set *read_set)
+cap_poll(fd_set *read_set
+#ifdef linux
+   _unused_
+#endif
+)
 {
    int total, ret;
 
 #ifndef linux /* We don't use select() on Linux. */
    if (!FD_ISSET(pcap_fd, read_set)) {
-      dverbosef("cap_poll premature");
+      verbosef("cap_poll premature");
       return;
    }
 #endif
@@ -202,7 +212,7 @@ cap_poll(fd_set *read_set)
    localip_update(); /* FIXME: this might even be too often */
 
    total = 0;
-   do {
+   for (;;) {
       ret = pcap_dispatch(
             pcap,
             -1,               /* count, -1 = entire buffer */
@@ -214,7 +224,15 @@ cap_poll(fd_set *read_set)
 
       /* Despite count = -1, Linux will only dispatch one packet at a time. */
       total += ret;
-   } while (ret != 0);
+
+#ifdef linux
+      /* keep looping until we've dispatched all the outstanding packets */
+      if (ret == 0) break;
+#else
+      /* we get them all on the first shot */
+      break;
+#endif
+   }
    /*FIXME*/fprintf(stderr, "%-20d\r", total);
 }
 
