@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * conv.c: convenience functions.
  *
@@ -32,8 +32,6 @@
 #include <time.h>
 #include <unistd.h>
 
-#define PRIVDROP_USER "nobody"
-#define CHROOT_DIR "/var/empty"
 #define PATH_DEVNULL "/dev/null"
 
 /* malloc() that exits on failure. */
@@ -177,6 +175,48 @@ split(const char delimiter, const char *str, int *num_chunks)
    #undef PUSH
 }
 
+/* Given an HTTP query string and a key to search for, return the value
+ * associated with it, or NULL if there is no such key or qs is NULL.
+ * The returned string needs to be freed.
+ *
+ * e.g.:
+ * qs = "sort=in&start=20";
+ * qs_get(sq, "sort") returns "in"
+ * qs_get(sq, "end") returns NULL
+ */
+char *
+qs_get(const char *qs, const char *key)
+{
+   size_t pos, qslen, keylen;
+
+   if (qs == NULL) return NULL;
+
+   qslen = strlen(qs);
+   keylen = strlen(key);
+   pos = 0;
+   while (pos < qslen) {
+      if (!(pos + keylen + 1 < qslen))
+         /* not enough room for "key" + "=" */
+         return NULL;
+      else {
+         if (str_starts_with(qs+pos, key) && qs[pos+keylen] == '=') {
+            /* found key= */
+            size_t start, end;
+
+            start = pos + keylen + 1;
+            for (end=start; end<qslen && qs[end] != '&'; end++)
+               ;
+            return split_string(qs, start, end);
+         } else {
+            /* didn't find key, skip to next & */
+            do { pos++; } while ((pos < qslen) && (qs[pos] != '&'));
+            pos++; /* skip the ampersand */
+         }
+      }
+   }
+   return NULL; /* not found */
+}
+
 static int lifeline[2] = { -1, -1 };
 static int fd_null = -1;
 
