@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * darkstat.c: program body.
  *
@@ -28,7 +28,10 @@
 #include <unistd.h>
 #include <pcap.h>
 
-static int running = 1;
+#include "now.h"
+time_t now;
+
+static volatile int running = 1;
 
 static void
 sig_shutdown(int signum _unused_)
@@ -54,8 +57,8 @@ parse_port(const char *str)
 static void
 usage(const char *arg0)
 {
-   printf("usage: %s [-d] -i interface [-p port] [-b bindaddr] [-f filter]\n",
-      arg0);
+   printf("usage: %s -i interface [-D] [-n] [-r] [-p port] [-b bindaddr] \\\n"
+   "\t[-f filter] [-l network/netmask]\n", arg0);
    exit(EXIT_FAILURE);
 }
 
@@ -68,18 +71,21 @@ main(int argc, char **argv)
    int maxconn = -1;
    const char *interface = NULL;
    const char *filter = NULL;
-   int ch, want_daemonize = 0;
+   int ch, want_daemonize = 1, want_promisc = 1, want_dns = 1;
 
    printf(PACKAGE_STRING " using libpcap %d.%d\n",
       PCAP_VERSION_MAJOR, PCAP_VERSION_MINOR);
 
-   if (close(STDIN_FILENO) == -1)
-      warn("close(stdin)");
-
-   while ((ch = getopt(argc, argv, "di:p:b:f:")) != -1) {
+   while ((ch = getopt(argc, argv, "Dnri:p:b:f:l:")) != -1) {
    switch (ch) {
-   case 'd':
-      want_daemonize = 1;
+   case 'D':
+      want_daemonize = 0;
+      break;
+   case 'n':
+      want_promisc = 0;
+      break;
+   case 'r':
+      want_dns = 0;
       break;
    case 'i':
       interface = optarg;
@@ -98,6 +104,9 @@ main(int argc, char **argv)
    case 'f':
       filter = optarg;
       break;
+   case 'l':
+      acct_init_localnet(optarg);
+      break;
    default:
       usage(argv[0]);
    }}
@@ -107,13 +116,19 @@ main(int argc, char **argv)
       usage(argv[0]);
    }
 
-   if (want_daemonize)
-      daemonize_start();
+   /* must verbosef() before first fork to init lock */
+   verbosef("starting up");
 
-   dns_init(); /* do this first as it forks - minimize memory use */
+   if (want_daemonize) {
+      verbosef("daemonizing to run in the background!");
+      daemonize_start();
+   }
+
+   if (want_dns)
+      dns_init(); /* do this first as it forks - minimize memory use */
 
    /* Need root privs for these: */
-   cap_init(interface, filter);
+   cap_init(interface, filter, want_promisc);
    http_init(bindaddr, bindport, maxconn);
    ncache_init(); /* don't need priv, but must do before chroot */
 
@@ -137,6 +152,8 @@ main(int argc, char **argv)
       struct timeval timeout;
       fd_set rs, ws;
 
+      now = time(NULL);
+
       FD_ZERO(&rs);
       FD_ZERO(&ws);
 
