@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * decode.c: packet decoding.
  *
@@ -54,6 +54,8 @@ static void decode_ether(u_char *, const struct pcap_pkthdr *,
    const u_char *);
 static void decode_loop(u_char *, const struct pcap_pkthdr *,
    const u_char *);
+static void decode_ppp(u_char *, const struct pcap_pkthdr *,
+   const u_char *);
 static void decode_pppoe(u_char *, const struct pcap_pkthdr *,
    const u_char *);
 static void decode_linux_sll(u_char *, const struct pcap_pkthdr *,
@@ -67,10 +69,15 @@ static const linkhdr_t linkhdrs[] = {
    { DLT_EN10MB,    ETHER_HDR_LEN, decode_ether  },
    { DLT_LOOP,      NULL_HDR_LEN,  decode_loop  },
    { DLT_NULL,      NULL_HDR_LEN,  decode_loop  },
-   { DLT_PPP,       PPP_HDR_LEN,   NULL },
+   { DLT_PPP,       PPP_HDR_LEN,   decode_ppp },
+#if defined(__NetBSD__)
+   { DLT_PPP_SERIAL, PPP_HDR_LEN, decode_ppp },
+#endif
    { DLT_FDDI,      FDDI_HDR_LEN,  NULL },
    { DLT_PPP_ETHER, PPPOE_HDR_LEN, decode_pppoe },
+#ifdef DLT_LINUX_SLL
    { DLT_LINUX_SLL, SLL_HDR_LEN,   decode_linux_sll },
+#endif
    { -1, -1, NULL }
 };
 
@@ -129,6 +136,14 @@ decode_ether(u_char *user _unused_,
       return;
    }
 
+#ifdef __sun
+   memcpy(sm.src_mac, hdr->ether_shost.ether_addr_octet, sizeof(sm.src_mac));
+   memcpy(sm.dst_mac, hdr->ether_dhost.ether_addr_octet, sizeof(sm.dst_mac));
+#else
+   memcpy(sm.src_mac, hdr->ether_shost, sizeof(sm.src_mac));
+   memcpy(sm.dst_mac, hdr->ether_dhost, sizeof(sm.dst_mac));
+#endif
+
    type = ntohs( hdr->ether_type );
    switch (type) {
    case ETHERTYPE_IP:
@@ -171,6 +186,22 @@ decode_loop(u_char *user _unused_,
       verbosef("loop: unknown family (%x)", family);
 }
 
+static void
+decode_ppp(u_char *user _unused_,
+      const struct pcap_pkthdr *pheader,
+      const u_char *pdata)
+{
+   pktsummary sm;
+   memset(&sm, 0, sizeof(sm));
+
+   if (pdata[2] == 0x00 && pdata[3] == 0x21) {
+         decode_ip(pdata + PPP_HDR_LEN, pheader->caplen - PPP_HDR_LEN, &sm);
+         sm.time = pheader->ts.tv_sec;
+         acct_for(&sm);
+   } else
+      verbosef("non-IP PPP packet; ignoring.");
+}
+
 static void
 decode_pppoe(u_char *user _unused_,
       const struct pcap_pkthdr *pheader,
