@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * err.c: BSD-like err() and warn() functions
  *
@@ -16,15 +16,16 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "darkstat.h"
 #include "err.h"
+#include <unistd.h>
+#include <stdarg.h>
+#include <stdio.h>
 
 #ifndef HAVE_ERR_H
 #include <errno.h>
-#include <stdarg.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
 void
 err(const int code, const char *format, ...)
@@ -78,4 +79,63 @@ warnx(const char *format, ...)
 
 #endif /* HAVE_ERR_H */
 
+/* We interlock verbosef() between processes by using a pipe with a single
+ * byte in it.  This pipe must be initialized before the first fork() in order
+ * to work.  Then, verbosef() will block on a read() until it is able to
+ * retrieve the byte.  After doing its business, it will put a byte back into
+ * the pipe.
+ *
+ * This is completely silly and largely unnecessary.
+ */
+static int inited = 0;
+static int lockpipe[2];
+
+static void unlock(void);
+
+static void
+initlock(void)
+{
+   if (pipe(lockpipe) == -1)
+      err(1, "pipe(lockpipe)");
+   inited = 1;
+   unlock();
+}
+
+static void
+lock(void)
+{
+   char buf[1];
+
+   if (!inited) initlock();
+   read(lockpipe[0], buf, 1);
+}
+
+static void
+unlock(void)
+{
+   char c = 0;
+
+   write(lockpipe[1], &c, 1);
+}
+
+void
+verbosef(const char *format, ...)
+{
+   va_list va;
+
+   lock();
+   fprintf(stderr, "darkstat (%05d): ", (int)getpid());
+   va_start(va, format);
+   vfprintf(stderr, format, va);
+   va_end(va);
+   fprintf(stderr, "\n");
+   unlock();
+}
+
+void
+dverbosef(const char *format _unused_, ...)
+{
+   /* disabled / do-nothing verbosef */
+}
+
 /* vim:set ts=3 sw=3 tw=78 expandtab: */
