@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * err.h: BSD-like err() and warn() functions
  *
@@ -29,4 +29,8 @@ void warn(const char *format, ...);
 void warnx(const char *format, ...);
 
 #endif
+
+void verbosef(const char *format, ...);
+void dverbosef(const char *format _unused_, ...);
+
 /* vim:set ts=3 sw=3 tw=78 expandtab: */
