@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2006 Emil Mikulic.
+ * copyright (c) 2006, 2007 Emil Mikulic.
  *
  * graph_db.c: round robin database for graph data
  *
@@ -14,11 +14,11 @@
 #include "str.h"
 #include "html.h" /* FIXME: should be pushed into a .c file? */
 #include "graph_db.h"
+#include "now.h"
 
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h> /* for memcpy() */
-#include <time.h>
 
 #define GRAPH_WIDTH "320"
 #define GRAPH_HEIGHT "200"
@@ -42,6 +42,8 @@ static struct graph *graph_db[] = {
 
 static unsigned int graph_db_size = sizeof(graph_db)/sizeof(*graph_db);
 
+static time_t start_time = 0;
+
 void
 graph_init(void)
 {
@@ -50,6 +52,7 @@ graph_init(void)
       graph_db[i]->in  = xcalloc(sizeof(uint64_t), graph_db[i]->num_bars);
       graph_db[i]->out = xcalloc(sizeof(uint64_t), graph_db[i]->num_bars);
    }
+   start_time = time(NULL);
 }
 
 void
@@ -165,7 +168,7 @@ graph_rotate(void)
    time_t t;
    struct tm *tm;
 
-   t = time(NULL);
+   t = now;
 
    if (last_time == 0) {
       verbosef("first rotate");
@@ -202,14 +205,52 @@ graph_rotate(void)
    advance(&graph_days, tm->tm_mday - 1);
 }
 
+/* ---------------------------------------------------------------------------
+ * Format a length of time in seconds to something more readable.
+ */
+static struct str *
+length_of_time(const time_t t)
+{
+   struct str *buf = str_make();
+   int secs  =  t % 60;
+   int mins  = (t / 60) % 60;
+   int hours = (t / 3600) % 24;
+   int days  =  t / 86400;
+
+   int show_zeroes = 0;
+
+   if (days > 0) {
+      str_appendf(buf, "%d %s", days, (days==1)?"day":"days");
+      show_zeroes = 1;
+   }
+
+   if (show_zeroes || (hours > 0)) {
+      if (show_zeroes) str_append(buf, ", ");
+      str_appendf(buf, "%d %s", hours, (hours==1)?"hr":"hrs");
+      show_zeroes = 1;
+   }
+
+   if (show_zeroes || (mins > 0)) {
+      if (show_zeroes) str_append(buf, ", ");
+      str_appendf(buf, "%d %s", mins, (mins==1)?"min":"mins");
+      show_zeroes = 1;
+   }
+
+   if (show_zeroes) str_append(buf, ", ");
+   str_appendf(buf, "%d %s", secs, (secs==1)?"sec":"secs");
+
+   return buf;
+}
+
 /* ---------------------------------------------------------------------------
  * Web interface: front page!
  */
 struct str *
 html_front_page(void)
 {
-   struct str *buf;
+   struct str *buf, *rf;
    unsigned int i;
+   char start_when[100];
 
    buf = str_make();
    str_append(buf, html_header_1);
@@ -219,17 +260,32 @@ html_front_page(void)
    str_append(buf, html_header_2);
    str_append(buf, "<h2 class=\"pageheader\">Graphs</h2>\n");
 
-   str_appendf(buf,
-      "<p>\n"
-       "<b>Total packets:</b> <span id=\"tp\">%'qu</span><br/>\n"
-       "<b>Total bytes:</b> <span id=\"tb\">%'qu</span><br/>\n"
-      "</p>\n",
-      total_packets, total_bytes);
+   str_append(buf, "<p>\n");
+
+   str_append(buf, "<b>Running for</b> <span id=\"rf\">");
+   rf = length_of_time(now - start_time);
+   /* FIXME: use a more monotonic clock perhaps? */
+   str_appendstr(buf, rf);
+   str_free(rf);
+   str_append(buf, "</span>");
+
+   if (strftime(start_when, sizeof(start_when),
+      "%Y-%m-%d %H:%M:%S %Z%z", localtime(&start_time)) != 0)
+      str_appendf(buf, "<b>, since</b> %s", start_when);
+
+   str_appendf(buf,"<b>.</b><br/>\n"
+      "<b>Total</b> <span id=\"tb\">%'qu</span> <b>bytes in</b> "
+      "<span id=\"tp\">%'qu</span> <b>packets.</b><br/>\n</p>\n"
+      ,total_bytes, total_packets);
 
    str_append(buf, "<div>\n");
-   for (i=0; i<graph_db_size; i++)
+   for (i=0; i<graph_db_size; i++) {
       str_appendf(buf,"<div class=\"outergraph\" id=\"g%d\">%s</div>\n",
          i, (i==0)?"Graphs require JavaScript.":"");
+      /* two above, break, two below */
+      if (i == 1)
+        str_append(buf, "<div style=\"clear:both\"></div>\n");
+   }
    str_append(buf,
       "<div style=\"clear:both\"></div>\n"
       "<div id=\"graph_buttons\"></div>\n" /* this gets filled out by JS */
@@ -272,10 +328,14 @@ struct str *
 xml_graphs(void)
 {
    unsigned int i, j;
-   struct str *buf = str_make();
+   struct str *buf = str_make(), *rf;
 
-   str_appendf(buf, "<graphs tp=\"%'qu\" tb=\"%'qu\">\n",
+   str_appendf(buf, "<graphs tp=\"%'qu\" tb=\"%'qu\" rf=\"",
       total_packets, total_bytes);
+   rf = length_of_time(now - start_time);
+   str_appendstr(buf, rf);
+   str_free(rf);
+   str_append(buf, "\">\n");
 
    for (i=0; i<graph_db_size; i++) {
       const struct graph *g = graph_db[i];
