@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * hosts_db.c: database of hosts, ports, protocols.
  *
@@ -18,10 +18,13 @@
 
 #include <arpa/inet.h> /* inet_aton() */
 #include <assert.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h> /* memset(), strcmp() */
 
+int show_mac_addrs = 0;
+
 /* FIXME: specify somewhere more sane/tunable */
 #define MAX_ENTRIES 30 /* in an HTML table rendered from a hashtable */
 
@@ -189,6 +192,7 @@ make_func_host(const void *key)
    MAKE_BUCKET(b, h, host);
    h->ip = CASTKEY(in_addr_t);
    h->dns = NULL;
+   memset(&h->mac_addr, 0, sizeof(h->mac_addr));
    h->ports_tcp = NULL;
    h->ports_udp = NULL;
    h->ip_protos = NULL;
@@ -227,16 +231,23 @@ make_func_ip_proto(const void *key)
 static void
 format_cols_host(struct str *buf)
 {
+   /* FIXME: don't clobber parts of the query string
+    * specifically "full" and "start"
+    * when setting sort direction
+    */
    str_append(buf,
       "<table border=\"1\">\n"
       "<tr>\n"
       " <th>IP</th>\n"
-      " <th>Hostname</th>\n"
-      " <th>In</th>\n"
-      " <th>Out</th>\n"
-      " <th>Total</th>\n"
-      "</tr>\n"
-   );
+      " <th>Hostname</th>\n");
+   if (show_mac_addrs)
+   str_append(buf,
+      " <th>MAC Address</th>\n");
+   str_append(buf,
+      " <th><a href=\"?sort=in\">In</a></th>\n"
+      " <th><a href=\"?sort=out\">Out</a></th>\n"
+      " <th><a href=\"?sort=total\">Total</a></th>\n"
+      "</tr>\n");
 }
 
 static void
@@ -244,6 +255,26 @@ format_row_host(struct str *buf, const struct bucket *b)
 {
    const char *ip = ip_to_str( b->u.host.ip );
 
+   if (show_mac_addrs)
+   str_appendf(buf,
+      "<tr>\n"
+      " <td><a href=\"%s/\">%s</a></td>\n"
+      " <td>%s</td>\n"
+      " <td><tt>%x:%x:%x:%x:%x:%x</tt></td>\n"
+      " <td>%'qu</td>\n"
+      " <td>%'qu</td>\n"
+      " <td>%'qu</td>\n"
+      "</tr>\n",
+      ip, ip, (b->u.host.dns == NULL)?"":b->u.host.dns,
+      b->u.host.mac_addr[0],
+      b->u.host.mac_addr[1],
+      b->u.host.mac_addr[2],
+      b->u.host.mac_addr[3],
+      b->u.host.mac_addr[4],
+      b->u.host.mac_addr[5],
+      b->in, b->out, b->total
+   );
+   else
    str_appendf(buf,
       "<tr>\n"
       " <td><a href=\"%s/\">%s</a></td>\n"
@@ -686,7 +717,7 @@ host_get_ip_proto(struct bucket *host, const uint16_t proto)
    return (hashtable_find_or_insert(h->ip_protos, &proto));
 }
 
-static struct str *html_hosts_main(void);
+static struct str *html_hosts_main(const char *qs);
 static struct str *html_hosts_detail(const char *ip);
 static struct str *html_hashtable_stats(struct hashtable *h);
 static struct str *html_tcp_stats(const char *ip);
@@ -697,7 +728,7 @@ static struct str *html_proto_stats(const char *ip);
  * Web interface: delegate the /hosts/ space.
  */
 struct str *
-html_hosts(const char *uri)
+html_hosts(const char *uri, const char *query)
 {
    int i, num_elems;
    char **elem = split('/', uri, &num_elems);
@@ -708,7 +739,7 @@ html_hosts(const char *uri)
 
    if (num_elems == 1)
       /* /hosts/ */
-      buf = html_hosts_main();
+      buf = html_hosts_main(query);
    else if ((num_elems == 2) && (strcmp(elem[1], "stats") == 0))
       /* /hosts/stats/ */
       buf = html_hashtable_stats(hosts_db);
@@ -736,10 +767,11 @@ html_hosts(const char *uri)
  * Format hashtable into HTML.
  */
 static void
-format_table(struct str *buf, struct hashtable *ht)
+format_table(struct str *buf, struct hashtable *ht, int start,
+   const enum sort_dir sort, const int full)
 {
    const struct bucket **table;
-   uint32_t i, pos, top;
+   uint32_t i, pos, end;
 
    if ((ht == NULL) || (ht->count == 0)) {
       str_append(buf, "<p>The table is empty.</p>\n");
@@ -757,12 +789,17 @@ format_table(struct str *buf, struct hashtable *ht)
    }
    assert(pos == ht->count);
 
-   top = min(ht->count, MAX_ENTRIES);
-   str_appendf(buf, "(top %u of %u)<br/>", top, ht->count); /* FIXME: ugly */
-   qsort_buckets(table, ht->count, 0, top, TOTAL
-      /* FIXME: offset(left), count(right), direction */ );
+   if (full) {
+      /* full report overrides start and end */
+      start = 0;
+      end = ht->count;
+   } else
+      end = min(ht->count, (uint32_t)start+MAX_ENTRIES);
+
+   str_appendf(buf, "(%u-%u of %u)<br/>", start+1, end, ht->count);
+   qsort_buckets(table, ht->count, start, end, sort);
    ht->format_cols_func(buf);
-   for (i=0; i<top; i++)
+   for (i=start; i<end; i++)
       ht->format_row_func(buf, table[i]);
    free(table);
    str_append(buf, "</table>\n");
@@ -772,20 +809,92 @@ format_table(struct str *buf, struct hashtable *ht)
  * Web interface: sorted table of hosts.
  */
 static struct str *
-html_hosts_main(void)
+html_hosts_main(const char *qs)
 {
-   struct str *buf;
-   assert(hosts_db != NULL);
+   struct str *buf = str_make();
+   char *qs_start, *qs_sort, *qs_full, *ep;
+   const char *sortstr;
+   int start, full = 0;
+   enum sort_dir sort;
+
+   /* parse query string */
+   qs_start = qs_get(qs, "start");
+   qs_sort = qs_get(qs, "sort");
+   qs_full = qs_get(qs, "full");
+   if (qs_full != NULL) {
+      full = 1;
+      free(qs_full);
+   }
+
+   /* validate sort */
+   if (qs_sort == NULL) sort = TOTAL;
+   else if (strcmp(qs_sort, "total") == 0) sort = TOTAL;
+   else if (strcmp(qs_sort, "in") == 0) sort = IN;
+   else if (strcmp(qs_sort, "out") == 0) sort = OUT;
+   else {
+      str_append(buf, "Error: invalid value for \"sort\".\n");
+      goto done;
+   }
+
+   /* parse start */
+   if (qs_start == NULL)
+      start = 0;
+   else {
+      start = (int)strtoul(qs_start, &ep, 10);
+      if (*ep != '\0') {
+         str_append(buf, "Error: \"start\" is not a number.\n");
+         goto done;
+      }
+      if ((errno == ERANGE) ||
+          (start < 0) || (start >= (int)hosts_db->count)) {
+         str_append(buf, "Error: \"start\" is out of bounds.\n");
+         goto done;
+      }
+   }
+
+#define PREV "&lt;&lt;&lt; prev page"
+#define NEXT "next page &gt;&gt;&gt;"
+#define FULL "full table"
 
-   buf = str_make();
    str_append(buf, html_header_1);
    str_append(buf, "<title>darkstat3: Hosts</title>\n");
    str_append(buf, html_header_2);
    str_append(buf, "<h2 class=\"pageheader\">Hosts</h2>\n");
-   format_table(buf, hosts_db);
-   str_append(buf, "<a href=\"stats/\">statistics</a><br/>\n");
+   format_table(buf, hosts_db, start, sort, full);
+
+   /* <prev | full | stats | next> */
+   sortstr = qs_sort;
+   if (sortstr == NULL) sortstr = "total";
+   if (start > 0) {
+      int prev = max(start - MAX_ENTRIES, 0);
+      str_appendf(buf, "<a href=\"?start=%d&sort=%s\">" PREV "</a>",
+         prev, sortstr);
+   } else
+      str_append(buf, PREV);
+
+   if (full)
+      str_append(buf, " | " FULL);
+   else
+      str_appendf(buf, " | <a href=\"?full=yes&sort=%s\">" FULL "</a>",
+         sortstr);
+
+   str_append(buf, " | <a href=\"stats/\">statistics</a> | ");
+
+   if (start+MAX_ENTRIES < (int)hosts_db->count)
+      str_appendf(buf, "<a href=\"?start=%d&sort=%s\">" NEXT "</a>",
+         start+MAX_ENTRIES, sortstr);
+   else
+      str_append(buf, NEXT);
+
+   str_append(buf, "<br/>\n");
    str_append(buf, html_footer);
-   return (buf);
+done:
+   if (qs_start != NULL) free(qs_start);
+   if (qs_sort != NULL) free(qs_sort);
+   return buf;
+#undef PREV
+#undef NEXT
+#undef FULL
 }
 
 /* ---------------------------------------------------------------------------
@@ -803,31 +912,46 @@ html_hosts_detail(const char *ip)
 
    /* Overview. */
    buf = str_make();
-   /* FIXME: dns name */
+   str_append(buf, html_header_1);
+   str_appendf(buf, " <title>%s</title>\n", ip);
+   str_append(buf, html_header_2);
    str_appendf(buf,
-      "%s"
-      " <title>%s</title>\n"
-      "%s"
       "<h2>%s</h2>\n"
       "<p>\n"
-      " <b>In:</b> %'qu<br>\n"
-      " <b>Out:</b> %'qu<br>\n"
-      " <b>Total:</b> %'qu<br>\n"
+       "<b>Hostname:</b> %s<br/>\n",
+      ip,
+      (h->u.host.dns == NULL)?"(resolving...)":h->u.host.dns);
+
+   if (show_mac_addrs)
+   str_appendf(buf,
+      "<b>MAC Address:</b> "
+      "<tt>%x:%x:%x:%x:%x:%x</tt><br/>\n",
+      h->u.host.mac_addr[0],
+      h->u.host.mac_addr[1],
+      h->u.host.mac_addr[2],
+      h->u.host.mac_addr[3],
+      h->u.host.mac_addr[4],
+      h->u.host.mac_addr[5]);
+
+   str_appendf(buf,
       "</p>\n"
-      ,
-      html_header_1, ip, html_header_2, ip, h->in, h->out, h->total
-   );
+      "<p>\n"
+      " <b>In:</b> %'qu<br/>\n"
+      " <b>Out:</b> %'qu<br/>\n"
+      " <b>Total:</b> %'qu<br/>\n"
+      "</p>\n",
+      h->in, h->out, h->total);
 
    str_append(buf, "<h3>TCP ports</h3>\n");
-   format_table(buf, h->u.host.ports_tcp);
+   format_table(buf, h->u.host.ports_tcp, 0,TOTAL,0);
    str_append(buf, "<a href=\"tcpstats/\">statistics</a><br/>\n");
 
    str_append(buf, "<h3>UDP ports</h3>\n");
-   format_table(buf, h->u.host.ports_udp);
+   format_table(buf, h->u.host.ports_udp, 0,TOTAL,0);
    str_append(buf, "<a href=\"udpstats/\">statistics</a><br/>\n");
 
    str_append(buf, "<h3>IP protocols</h3>\n");
-   format_table(buf, h->u.host.ip_protos);
+   format_table(buf, h->u.host.ip_protos, 0,TOTAL,0);
    str_append(buf, "<a href=\"protostats/\">statistics</a><br/>\n");
 
    str_append(buf, html_footer);
@@ -881,18 +1005,18 @@ html_hashtable_stats(struct hashtable *h)
 
       str_appendf(buf,
          "<p>\n"
-         "<b>Slots:</b> %'u<br>\n"
-         "<b>Records:</b> %'u<br>\n"
-         "<b>Slots empty:</b> %'u<br>\n"
-         "<b>Slots overflowed:</b> %'u<br>\n"
-         "<b>Overflow buckets:</b> %'u<br>\n"
-         "<b>Longest chain:</b> %'u<br>\n"
-         "<b>Usage:</b> %u%%<br>\n"
-         "<br>\n"
-         "<b>Inserts:</b> %'qu<br>\n"
-         "<b>Searches:</b> %'qu<br>\n"
-         "<b>Deletions:</b> %'qu<br>\n"
-         "<b>Rehashes:</b> %'qu<br>\n"
+         "<b>Slots:</b> %'u<br/>\n"
+         "<b>Records:</b> %'u<br/>\n"
+         "<b>Slots empty:</b> %'u<br/>\n"
+         "<b>Slots overflowed:</b> %'u<br/>\n"
+         "<b>Overflow buckets:</b> %'u<br/>\n"
+         "<b>Longest chain:</b> %'u<br/>\n"
+         "<b>Usage:</b> %u%%<br/>\n"
+         "<br/>\n"
+         "<b>Inserts:</b> %'qu<br/>\n"
+         "<b>Searches:</b> %'qu<br/>\n"
+         "<b>Deletions:</b> %'qu<br/>\n"
+         "<b>Rehashes:</b> %'qu<br/>\n"
          "</p>\n"
          ,
          h->size, h->count, num_empty, ovr_slots, ovr_buckets, max_len,
