@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * hosts_db.h: database of hosts, ports, protocols.
  *
@@ -16,6 +16,7 @@ struct hashtable;
 struct host {
    in_addr_t ip;
    char *dns;
+   uint8_t mac_addr[6];
    struct hashtable *ports_tcp, *ports_udp, *ip_protos;
 };
 
@@ -45,6 +46,8 @@ struct bucket {
 
 enum sort_dir { IN, OUT, TOTAL };
 
+extern int show_mac_addrs;
+
 void hosts_db_init(void);
 void hosts_db_reduce(void);
 void hosts_db_free(void);
@@ -56,7 +59,7 @@ struct bucket *host_get_port_udp(struct bucket *host, const uint16_t port);
 struct bucket *host_get_ip_proto(struct bucket *host, const uint16_t proto);
 
 /* Web pages. */
-struct str *html_hosts(const char *uri);
+struct str *html_hosts(const char *uri, const char *query);
 
 /* From hosts_sort */
 void qsort_buckets(const struct bucket **a, size_t n,
